import { fetchCDNConfig } from './requests';
import { generateBucketedConfig } from '@devcycle/bucketing';
import { getIdentity, getSDKKey } from './requestContext';
import { cache } from 'react';
import { DVCPopulatedUser } from '@devcycle/js-client-sdk';
import { sseURlGetter } from './ably';
import { getUserAgent } from './userAgent';
// wrap this function in react cache to avoid redoing work for the same user and config
const generateBucketedConfigCached = cache(async (user, configResponse) => {
    var _a;
    const config = await configResponse.json();
    const ua = getUserAgent();
    const populatedUser = new DVCPopulatedUser(user, {}, undefined, undefined, ua !== null && ua !== void 0 ? ua : undefined);
    return {
        config: {
            ...generateBucketedConfig({ user: populatedUser, config }),
            sse: {
                url: await sseURlGetter(getSDKKey(), (_a = config.ably) === null || _a === void 0 ? void 0 : _a.apiKey)(),
                inactivityDelay: 1000 * 60 * 2,
            },
        },
    };
});
/**
 * Retrieve the config from CDN for the current request's SDK Key. This data will often be cached
 * Compute the bucketed config for the current request's user using that data, with local bucketing library
 * Cache the bucketed config for this request so that repeated calls to this function are memoized
 */
export const getBucketedConfig = async () => {
    var _a;
    // this request will be cached by Next
    const cdnConfig = await fetchCDNConfig(getSDKKey());
    if (!cdnConfig.ok) {
        const responseText = await cdnConfig.text();
        throw new Error('Could not fetch config: ' + responseText);
    }
    const user = getIdentity();
    if (!user) {
        throw Error('User should be defined');
    }
    const { config } = await generateBucketedConfigCached(user, cdnConfig);
    return {
        ...config,
        lastModified: (_a = cdnConfig.headers.get('last-modified')) !== null && _a !== void 0 ? _a : undefined,
    };
};
//# sourceMappingURL=bucketing.js.map